@@ -12,7 +12,7 @@ Python libraries, though it does make heavy use of external packaging commands.
 Visit <https://github.com/datalad/datalad-installer> for more information.
 """
 
-__version__ = "0.8.2"
+__version__ = "0.9.1"
 __author__ = "The DataLad Team and Contributors"
 __author_email__ = "team@datalad.org"
 __license__ = "MIT"
@@ -373,9 +373,13 @@ class OptionParser:
         lines = [self.short_help(progname)]
         if self.help is not None:
             lines.append("")
-            lines.extend(
-                " " * HELP_INDENT + ln for ln in textwrap.wrap(self.help, HELP_WIDTH)
-            )
+            for ln in self.help.splitlines():
+                if ln == "":
+                    lines.append("")
+                else:
+                    lines.extend(
+                        " " * HELP_INDENT + wl for wl in textwrap.wrap(ln, HELP_WIDTH)
+                    )
         if self.options:
             lines.append("")
             lines.append("Options:")
@@ -461,8 +465,61 @@ class CondaInstance(NamedTuple):
             return self.basepath / "envs" / self.name / dirname
 
 
-#: A list of command names and the paths at which they are located
-CommandList = List[Tuple[str, Path]]
+class Command:
+    """An external command that can be installed by this script"""
+
+    def __init__(self, name: str, test_args: Optional[List[str]] = None) -> None:
+        #: Name of the command
+        self.name = name
+        if test_args is None:
+            #: Arguments with which to invoke the command as a smoke test
+            self.test_args = ["--help"]
+        else:
+            self.test_args = test_args
+
+    def in_bindir(self, bindir: Path) -> "InstalledCommand":
+        """
+        Return an `InstalledCommand` recording that the command is installed in
+        the given directory
+        """
+        cmdpath = bindir / self.name
+        if ON_WINDOWS and cmdpath.suffix == "":
+            cmdpath = cmdpath.with_suffix(".exe")
+        return InstalledCommand(name=self.name, path=cmdpath, test_args=self.test_args)
+
+
+class InstalledCommand(Command):
+    """An external command that has been installed by this script"""
+
+    def __init__(
+        self, name: str, path: Path, test_args: Optional[List[str]] = None
+    ) -> None:
+        super().__init__(name, test_args)
+        #: Path at which the command is installed
+        self.path = path
+
+    def test(self) -> bool:
+        test_cmd = [str(self.path)] + self.test_args
+        cmdtext = " ".join(map(shlex.quote, test_cmd))
+        try:
+            sr = subprocess.run(test_cmd, stdout=subprocess.DEVNULL)
+        except Exception as e:
+            log.error("Failed to run `%s`: %s", cmdtext, e)
+            return False
+        else:
+            if sr.returncode != 0:
+                log.error("`%s` command failed!", cmdtext)
+                return False
+            else:
+                return True
+
+
+DATALAD_CMD = Command("datalad")
+GIT_ANNEX_CMD = Command("git-annex")
+# Smoke-test rclone with --version instead of --help because the latter didn't
+# exist before rclone 1.33
+RCLONE_CMD = Command("rclone", test_args=["--version"])
+GIT_ANNEX_REMOTE_RCLONE_CMD = Command("git-annex-remote-rclone")
 
 
 class DataladInstaller:
@@ -471,7 +528,15 @@ class DataladInstaller:
     COMPONENTS: ClassVar[Dict[str, Type["Component"]]] = {}
 
     OPTION_PARSER = OptionParser(
-        help="Installation script for Datalad and related components",
+        help=(
+            "Installation script for Datalad and related components\n\n"
+            "`datalad-installer` is a script for installing Datalad, git-annex,"
+            " and related components all in a single invocation.  It requires"
+            " no third-party Python libraries, though it does make heavy use of"
+            " external packaging commands.\n\n"
+            "See the README at <https://github.com/datalad/datalad-installer>"
+            " for a complete description of all options."
+        ),
         options=[
             Option(
                 "-V",
@@ -536,7 +601,7 @@ class DataladInstaller:
         #: instance
         self.conda_stack: List[CondaInstance] = []
         #: A list of commands installed via the instance
-        self.new_commands: CommandList = []
+        self.new_commands: List[InstalledCommand] = []
         #: Whether "brew update" has been run
         self.brew_updated: bool = False
 
@@ -691,26 +756,16 @@ class DataladInstaller:
         for cr in components:
             self.addcomponent(name=cr.name, **cr.kwargs)
         ok = True
-        for name, path in self.new_commands:
-            log.info("%s is now installed at %s", name, path)
-            if not os.path.exists(path):
-                log.error("%s does not exist!", path)
+        for cmd in self.new_commands:
+            log.info("%s is now installed at %s", cmd.name, cmd.path)
+            if not os.path.exists(cmd.path):
+                log.error("%s does not exist!", cmd.path)
                 ok = False
-            elif not ON_WINDOWS and not os.access(path, os.X_OK):
-                log.error("%s is not executable!", path)
+            elif not ON_WINDOWS and not os.access(cmd.path, os.X_OK):
+                log.error("%s is not executable!", cmd.path)
+                ok = False
+            elif not cmd.test():
                 ok = False
-            else:
-                try:
-                    sr = subprocess.run(
-                        [str(path), "--help"], stdout=subprocess.DEVNULL
-                    )
-                except Exception as e:
-                    log.error("Failed to run `%s --help`: %s", path, e)
-                    ok = False
-                else:
-                    if sr.returncode != 0:
-                        log.error("`%s --help` command failed!", path)
-                        ok = False
         return 0 if ok else 1
 
     def addenv(self, line: str) -> None:
@@ -895,6 +950,11 @@ class MinicondaComponent(Component):
                     " the Miniconda environment"
                 ),
             ),
+            Option(
+                "--python-match",
+                choices=["major", "minor", "micro"],
+                help="Install the same version of Python, matching to the given version level",
+            ),
             Option(
                 "-e",
                 "--extra-args",
@@ -909,6 +969,7 @@ class MinicondaComponent(Component):
         path: Optional[Path] = None,
         batch: bool = False,
         spec: Optional[List[str]] = None,
+        python_match: Optional[str] = None,
         extra_args: Optional[List[str]] = None,
         **kwargs: Any,
     ) -> None:
@@ -929,6 +990,7 @@ class MinicondaComponent(Component):
         else:
             log.info("Batch: %s", batch)
         log.info("Spec: %s", spec)
+        log.info("Python Match: %s", python_match)
         log.info("Extra args: %s", extra_args)
         if kwargs:
             log.warning("Ignoring extra component arguments: %r", kwargs)
@@ -940,6 +1002,21 @@ class MinicondaComponent(Component):
             miniconda_script = "Miniconda3-latest-Windows-x86_64.exe"
         else:
             raise RuntimeError(f"E: Unsupported OS: {SYSTEM}")
+        if python_match is not None:
+            vparts: Tuple[int, ...]
+            if python_match == "major":
+                vparts = sys.version_info[:1]
+            elif python_match == "minor":
+                vparts = sys.version_info[:2]
+            elif python_match == "micro":
+                vparts = sys.version_info[:3]
+            else:
+                raise AssertionError(f"Unexpected python_match value: {python_match!r}")
+            newspec = f"python={'.'.join(map(str, vparts))}"
+            log.debug("Adding %r to spec", newspec)
+            if spec is None:
+                spec = []
+            spec.append(newspec)
         log.info("Downloading and running miniconda installer")
         with tempfile.TemporaryDirectory() as tmpdir:
             script_path = os.path.join(tmpdir, miniconda_script)
@@ -973,7 +1050,11 @@ class MinicondaComponent(Component):
                 runcmd("bash", script_path, *args)
         conda_instance = CondaInstance(basepath=path, name=None)
         if spec is not None:
-            runcmd(conda_instance.conda_exe, "install", *spec)
+            install_args: List[str] = []
+            if batch:
+                install_args.append("--yes")
+            install_args.extend(spec)
+            runcmd(conda_instance.conda_exe, "install", *install_args)
         self.manager.conda_stack.append(conda_instance)
         self.manager.installer_stack.append(
             CondaInstaller(self.manager, conda_instance)
@@ -1264,12 +1345,12 @@ class Installer(ABC):
 
     #: Mapping from supported installable component names to
     #: (installer-specific package IDs, list of installed programs) pairs
-    PACKAGES: ClassVar[Dict[str, Tuple[str, List[str]]]]
+    PACKAGES: ClassVar[Dict[str, Tuple[str, List[Command]]]]
 
     def __init__(self, manager: DataladInstaller) -> None:
         self.manager = manager
 
-    def install(self, component: str, **kwargs: Any) -> CommandList:
+    def install(self, component: str, **kwargs: Any) -> List[InstalledCommand]:
         """
         Installs a given component.  Raises `MethodNotSupportedError` if the
         installation method is not supported on the system or the method does
@@ -1284,13 +1365,7 @@ class Installer(ABC):
                 f"{self.NAME} does not know how to install {component}"
             )
         bindir = self.install_package(package, **kwargs)
-        bins = []
-        for cmd in commands:
-            p = bindir / cmd
-            if ON_WINDOWS and p.suffix == "":
-                p = p.with_suffix(".exe")
-            bins.append((cmd, p))
-        return bins
+        return [cmd.in_bindir(bindir) for cmd in commands]
 
     @abstractmethod
     def install_package(self, package: str, **kwargs: Any) -> Path:
@@ -1334,12 +1409,12 @@ class AptInstaller(Installer):
     ]
 
     PACKAGES = {
-        "datalad": ("datalad", ["datalad"]),
-        "git-annex": ("git-annex", ["git-annex"]),
-        "rclone": ("rclone", ["rclone"]),
+        "datalad": ("datalad", [DATALAD_CMD]),
+        "git-annex": ("git-annex", [GIT_ANNEX_CMD]),
+        "rclone": ("rclone", [RCLONE_CMD]),
         "git-annex-remote-rclone": (
             "git-annex-remote-rclone",
-            ["git-annex-remote-rclone"],
+            [GIT_ANNEX_REMOTE_RCLONE_CMD],
         ),
     }
 
@@ -1391,12 +1466,12 @@ class HomebrewInstaller(Installer):
     ]
 
     PACKAGES = {
-        "datalad": ("datalad", ["datalad"]),
-        "git-annex": ("git-annex", ["git-annex"]),
-        "rclone": ("rclone", ["rclone"]),
+        "datalad": ("datalad", [DATALAD_CMD]),
+        "git-annex": ("git-annex", [GIT_ANNEX_CMD]),
+        "rclone": ("rclone", [RCLONE_CMD]),
         "git-annex-remote-rclone": (
             "git-annex-remote-rclone",
-            ["git-annex-remote-rclone"],
+            [GIT_ANNEX_REMOTE_RCLONE_CMD],
         ),
     }
 
@@ -1451,7 +1526,7 @@ class PipInstaller(Installer):
     ]
 
     PACKAGES = {
-        "datalad": ("datalad", ["datalad"]),
+        "datalad": ("datalad", [DATALAD_CMD]),
     }
 
     DEVEL_PACKAGES = {
@@ -1545,7 +1620,7 @@ class NeurodebianInstaller(AptInstaller):
     NAME = "neurodebian"
 
     PACKAGES = {
-        "git-annex": ("git-annex-standalone", ["git-annex"]),
+        "git-annex": ("git-annex-standalone", [GIT_ANNEX_CMD]),
     }
 
     def assert_supported_system(self) -> None:
@@ -1575,12 +1650,12 @@ class DebURLInstaller(Installer):
     ]
 
     PACKAGES = {
-        "git-annex": ("git-annex", ["git-annex"]),
-        "datalad": ("datalad", ["datalad"]),
-        "rclone": ("rclone", ["rclone"]),
+        "git-annex": ("git-annex", [GIT_ANNEX_CMD]),
+        "datalad": ("datalad", [DATALAD_CMD]),
+        "rclone": ("rclone", [RCLONE_CMD]),
         "git-annex-remote-rclone": (
             "git-annex-remote-rclone",
-            ["git-annex-remote-rclone"],
+            [GIT_ANNEX_REMOTE_RCLONE_CMD],
         ),
     }
 
@@ -1632,7 +1707,7 @@ class AutobuildSnapshotInstaller(Installer):
     OPTIONS: ClassVar[List[Option]] = []
 
     PACKAGES = {
-        "git-annex": ("git-annex", ["git-annex"]),
+        "git-annex": ("git-annex", [GIT_ANNEX_CMD]),
     }
 
     def _install_linux(self, path: str) -> Path:
@@ -1720,9 +1795,9 @@ class CondaInstaller(Installer):
     ]
 
     PACKAGES = {
-        "datalad": ("datalad", ["datalad"]),
-        "git-annex": ("git-annex", ["git-annex"]),
-        "rclone": ("rclone", ["rclone"]),
+        "datalad": ("datalad", [DATALAD_CMD]),
+        "git-annex": ("git-annex", [GIT_ANNEX_CMD]),
+        "rclone": ("rclone", [RCLONE_CMD]),
     }
 
     def __init__(
@@ -1807,7 +1882,7 @@ class DataladGitAnnexBuildInstaller(Installer):
     ]
 
     PACKAGES = {
-        "git-annex": ("git-annex", ["git-annex"]),
+        "git-annex": ("git-annex", [GIT_ANNEX_CMD]),
     }
 
     VERSIONED = False
@@ -1934,7 +2009,7 @@ class DataladPackagesBuildInstaller(Installer):
     OPTIONS: ClassVar[List[Option]] = []
 
     PACKAGES = {
-        "git-annex": ("git-annex", ["git-annex"]),
+        "git-annex": ("git-annex", [GIT_ANNEX_CMD]),
     }
 
     def install_package(
@@ -1988,7 +2063,7 @@ class DMGInstaller(Installer):
     ]
 
     PACKAGES = {
-        "git-annex": ("git-annex", ["git-annex"]),
+        "git-annex": ("git-annex", [GIT_ANNEX_CMD]),
     }
 
     def install_package(
@@ -2030,7 +2105,7 @@ class GARRCGitHubInstaller(Installer):
     PACKAGES = {
         "git-annex-remote-rclone": (
             "git-annex-remote-rclone",
-            ["git-annex-remote-rclone"],
+            [GIT_ANNEX_REMOTE_RCLONE_CMD],
         ),
     }
 
@@ -2093,7 +2168,7 @@ class DownloadsRCloneInstaller(Installer):
         ),
     ]
 
-    PACKAGES = {"rclone": ("rclone", ["rclone"])}
+    PACKAGES = {"rclone": ("rclone", [RCLONE_CMD])}
 
     def install_package(
         self,
