@@ -58,6 +58,38 @@ def test_install_miniconda(tmp_path: Path) -> None:
     )
 
 
+@pytest.mark.skipif(
+    sys.version_info[:2] < (3, 7),
+    reason="Trying to install Python 3.6 on Conda gives package conflicts",
+)
+@pytest.mark.miniconda
+def test_install_miniconda_python_match(tmp_path: Path) -> None:
+    miniconda_path = tmp_path / "conda"
+    r = main(
+        [
+            "datalad_installer.py",
+            "miniconda",
+            "--batch",
+            "--path",
+            str(miniconda_path),
+            "--python-match",
+            "minor",
+        ]
+    )
+    assert r == 0
+    if ON_WINDOWS:
+        pypath = miniconda_path / "python.exe"
+    else:
+        pypath = miniconda_path / "bin" / "python"
+    assert pypath.exists()
+    assert subprocess.run(
+        [str(pypath), "-c", "import sys; print(sys.version_info[:2])"],
+        stdout=subprocess.PIPE,
+        universal_newlines=True,
+        check=True,
+    ).stdout.strip() == repr(sys.version_info[:2])
+
+
 @pytest.mark.miniconda
 def test_install_miniconda_autogen_path(monkeypatch: pytest.MonkeyPatch) -> None:
     # Override TMPDIR with a path that will be cleaned up afterwards (We can't
@@ -552,6 +584,11 @@ def test_install_latest_rclone_from_downloads_with_manpage(tmp_path: Path) -> No
             44712960,
             marks=pytest.mark.skipif(not ON_WINDOWS, reason="Windows only"),
         ),
+        pytest.param(
+            "1.30",
+            12462464,
+            marks=pytest.mark.skipif(not ON_LINUX, reason="Linux only"),
+        ),
     ],
 )
 def test_install_rclone_version_from_downloads(
